Linux LED subsystem development
 help / color / mirror / Atom feed
* [PATCH v2] leds: class: Use firmware nodes for device lookup
@ 2026-05-13 11:58 Alban Bedel
  2026-05-20 15:22 ` Lee Jones
  0 siblings, 1 reply; 2+ messages in thread
From: Alban Bedel @ 2026-05-13 11:58 UTC (permalink / raw)
  To: linux-leds; +Cc: Pavel Machek, Lee Jones, linux-kernel, Alban Bedel

Replace the OF based lookup with the fwnode equivalent to get support
for ACPI and software nodes.

Signed-off-by: Alban Bedel <alban.bedel@lht.dlh.de>
---
v2: * Keep the doc string
    * Update comment to reference the function now used
---
 drivers/leds/led-class.c | 30 ++++++++++++++++--------------
 1 file changed, 16 insertions(+), 14 deletions(-)

diff --git a/drivers/leds/led-class.c b/drivers/leds/led-class.c
index 9e14ae588f78..a17db3d6644f 100644
--- a/drivers/leds/led-class.c
+++ b/drivers/leds/led-class.c
@@ -249,32 +249,34 @@ static const struct class leds_class = {
 };
 
 /**
  * of_led_get() - request a LED device via the LED framework
  * @np: device node to get the LED device from
  * @index: the index of the LED
  * @name: the name of the LED used to map it to its function, if present
  *
  * Returns the LED device parsed from the phandle specified in the "leds"
  * property of a device tree node or a negative error-code on failure.
  */
-static struct led_classdev *of_led_get(struct device_node *np, int index,
-				       const char *name)
+static struct led_classdev *fwnode_led_get(struct fwnode_handle *fwnode,
+					   int index, const char *name)
 {
+	struct fwnode_handle *led_node;
 	struct device *led_dev;
-	struct device_node *led_node;
 
 	/*
 	 * For named LEDs, first look up the name in the "led-names" property.
 	 * If it cannot be found, then of_parse_phandle() will propagate the error.
 	 */
 	if (name)
-		index = of_property_match_string(np, "led-names", name);
-	led_node = of_parse_phandle(np, "leds", index);
-	if (!led_node)
-		return ERR_PTR(-ENOENT);
+		index = fwnode_property_match_string(fwnode, "led-names",
+						     name);
+	led_node = fwnode_find_reference(fwnode, "leds", index);
+	if (IS_ERR(led_node))
+		return ERR_CAST(led_node);
 
-	led_dev = class_find_device_by_fwnode(&leds_class, of_fwnode_handle(led_node));
-	of_node_put(led_node);
+	led_dev = class_find_device_by_fwnode(&leds_class, led_node);
+	fwnode_handle_put(led_node);
 
 	return led_module_get(led_dev);
 }
@@ -332,7 +334,7 @@ struct led_classdev *__must_check devm_of_led_get(struct device *dev,
 	if (!dev)
 		return ERR_PTR(-EINVAL);
 
-	led = of_led_get(dev->of_node, index, NULL);
+	led = fwnode_led_get(dev_fwnode(dev), index, NULL);
 	if (IS_ERR(led))
 		return led;
 
@@ -354,7 +356,7 @@ struct led_classdev *led_get(struct device *dev, char *con_id)
 	const char *provider = NULL;
 	struct device *led_dev;
 
-	led_cdev = of_led_get(dev->of_node, -1, con_id);
+	led_cdev = fwnode_led_get(dev_fwnode(dev), -1, con_id);
 	if (!IS_ERR(led_cdev) || PTR_ERR(led_cdev) != -ENOENT)
 		return led_cdev;
 
@@ -249,32 +249,34 @@ static const struct class leds_class = {
 
 /**
- * of_led_get() - request a LED device via the LED framework
- * @np: device node to get the LED device from
+ * fwnode_led_get() - request a LED device via the LED framework
+ * @fwnode: firmware node to get the LED device from
  * @index: the index of the LED
  * @name: the name of the LED used to map it to its function, if present
@@ -249,32 +249,34 @@ static const struct class leds_class = {
 	/*
 	 * For named LEDs, first look up the name in the "led-names" property.
-	 * If it cannot be found, then of_parse_phandle() will propagate the error.
+	 * If it cannot be found, then fwnode_find_reference() will propagate
+	 * the error.
 	 */
 	if (name)
-- 
2.39.5


^ permalink raw reply related	[flat|nested] 2+ messages in thread

* Re: [PATCH v2] leds: class: Use firmware nodes for device lookup
  2026-05-13 11:58 [PATCH v2] leds: class: Use firmware nodes for device lookup Alban Bedel
@ 2026-05-20 15:22 ` Lee Jones
  0 siblings, 0 replies; 2+ messages in thread
From: Lee Jones @ 2026-05-20 15:22 UTC (permalink / raw)
  To: Alban Bedel; +Cc: linux-leds, Pavel Machek, linux-kernel

On Wed, 13 May 2026, Alban Bedel wrote:

> Replace the OF based lookup with the fwnode equivalent to get support
> for ACPI and software nodes.
> 
> Signed-off-by: Alban Bedel <alban.bedel@lht.dlh.de>
> ---
> v2: * Keep the doc string
>     * Update comment to reference the function now used
> ---
>  drivers/leds/led-class.c | 30 ++++++++++++++++--------------
>  1 file changed, 16 insertions(+), 14 deletions(-)
> 
> diff --git a/drivers/leds/led-class.c b/drivers/leds/led-class.c
> index 9e14ae588f78..a17db3d6644f 100644
> --- a/drivers/leds/led-class.c
> +++ b/drivers/leds/led-class.c
> @@ -249,32 +249,34 @@ static const struct class leds_class = {
>  };
>  
>  /**
> - * of_led_get() - request a LED device via the LED framework
> - * @np: device node to get the LED device from
> + * fwnode_led_get() - request a LED device via the LED framework
> + * @fwnode: firmware node to get the LED device from
>   * @index: the index of the LED
>   * @name: the name of the LED used to map it to its function, if present
>   *
>   * Returns the LED device parsed from the phandle specified in the "leds"
>   * property of a device tree node or a negative error-code on failure.
>   */
> -static struct led_classdev *of_led_get(struct device_node *np, int index,
> -				       const char *name)
> +static struct led_classdev *fwnode_led_get(struct fwnode_handle *fwnode,
> +					   int index, const char *name)
>  {
> +	struct fwnode_handle *led_node;
>  	struct device *led_dev;
> -	struct device_node *led_node;
>  
>  	/*
>  	 * For named LEDs, first look up the name in the "led-names" property.
> -	 * If it cannot be found, then of_parse_phandle() will propagate the error.
> +	 * If it cannot be found, then fwnode_find_reference() will propagate
> +	 * the error.
>  	 */
>  	if (name)
> -		index = of_property_match_string(np, "led-names", name);
> -	led_node = of_parse_phandle(np, "leds", index);
> -	if (!led_node)
> -		return ERR_PTR(-ENOENT);
> +		index = fwnode_property_match_string(fwnode, "led-names",
> +						     name);
> +	led_node = fwnode_find_reference(fwnode, "leds", index);

What happens if fwnode_property_match_string() returns an error?

> +	if (IS_ERR(led_node))
> +		return ERR_CAST(led_node);
>  
> -	led_dev = class_find_device_by_fwnode(&leds_class, of_fwnode_handle(led_node));
> -	of_node_put(led_node);
> +	led_dev = class_find_device_by_fwnode(&leds_class, led_node);
> +	fwnode_handle_put(led_node);
>  
>  	return led_module_get(led_dev);
>  }
> @@ -332,7 +334,7 @@ struct led_classdev *__must_check devm_of_led_get(struct device *dev,
>  	if (!dev)
>  		return ERR_PTR(-EINVAL);
>  
> -	led = of_led_get(dev->of_node, index, NULL);
> +	led = fwnode_led_get(dev_fwnode(dev), index, NULL);
>  	if (IS_ERR(led))
>  		return led;
>  
> @@ -354,7 +356,7 @@ struct led_classdev *led_get(struct device *dev, char *con_id)
>  	const char *provider = NULL;
>  	struct device *led_dev;
>  
> -	led_cdev = of_led_get(dev->of_node, -1, con_id);
> +	led_cdev = fwnode_led_get(dev_fwnode(dev), -1, con_id);
>  	if (!IS_ERR(led_cdev) || PTR_ERR(led_cdev) != -ENOENT)
>  		return led_cdev;
>  
> -- 
> 2.39.5
> 

-- 
Lee Jones

^ permalink raw reply	[flat|nested] 2+ messages in thread

end of thread, other threads:[~2026-05-20 15:22 UTC | newest]

Thread overview: 2+ messages (download: mbox.gz follow: Atom feed
-- links below jump to the message on this page --
2026-05-13 11:58 [PATCH v2] leds: class: Use firmware nodes for device lookup Alban Bedel
2026-05-20 15:22 ` Lee Jones

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox